package se.gawell.setrace.jersey;

/*
 *     This file is part of SeTrace.
 *
 *   SeTrace is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SeTrace is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SeTrace.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import java.util.concurrent.atomic.AtomicLong;

import org.slf4j.MDC;

import com.sun.jersey.spi.container.ContainerRequest;
import com.sun.jersey.spi.container.ContainerRequestFilter;

/**
 * Request filter that reads the http header for a request trace id.
 * If there is no request trace id present in the header one will be generated from
 * the request url.
 * 
 * The resulting request trace id is then added to MDC.
 * 
 * @author Niklas Gawell
 *
 */
public class ServletRequestTraceIdRequestFilter implements ContainerRequestFilter {
	
	protected AtomicLong counter = new AtomicLong(System.currentTimeMillis());
	
	@Override
	public ContainerRequest filter(ContainerRequest request) {
		String requestTraceId = request.getHeaderValue(MDCKeys.REQUEST_TRACE_HEADER_NAME);
		if (requestTraceId == null || requestTraceId.equals("")) {
			requestTraceId = generateRequestTraceId(request);
		}
		MDC.put(MDCKeys.REQUEST_TRACE_ID_NAME, requestTraceId);
		return request;
	}

	protected String generateRequestTraceId(ContainerRequest request) {
		return request.getPath(true) + "_" + counter.incrementAndGet();
	}

}
